@@ -36,12 +36,17 @@ Mie scattering intensities normalized to one when integrated over all angles::
     miepython.i_per(m, x, mu, norm='one')
     miepython.i_par(m, x, mu, norm='one')
     miepython.i_unpolarized(m, x, mu, norm='one')
+
+The scattering matrix
+
+    miepython.mie_phase_matrix(m, x, mu)
+
 """
-__version__ = '2.3.2'
+__version__ = '2.5.4'
 __author__ = 'Scott Prahl'
 __email__ = 'scott.prahl@oit.edu'
-__copyright__ = 'Copyright 2017-23, Scott Prahl'
+__copyright__ = '2017-24, Scott Prahl'
 __license__ = 'MIT'
-__url__ = 'https://github.com/scottprahl/miepython.git'
+__url__ = 'https://github.com/scottprahl/miepython'
 
 from .miepython import *
