@@ -1,10 +1,6 @@
 #!/usr/bin/env python3
 
-"""
-Plot the scattering cross section for 100nm gold spheres.
-
-The resulting graph is as a function of wavelength.
-"""
+"""Plot the scattering cross section for 100nm gold spheres."""
 
 import numpy as np
 import matplotlib.pyplot as plt
@@ -42,25 +38,24 @@ qext, qsca, qback, g = miepython.mie(m, x)
 sca_cross_section = qsca * cross_section_area
 abs_cross_section = (qext - qsca) * cross_section_area
 
-plt.subplots(3, 1, figsize=(9, 9))
+plt.subplots(3, 1, figsize=(8, 8))
 plt.subplot(311)
-plt.plot(ref_lam * 1000, ref_n, 'ob')
-plt.plot(ref_lam * 1000, -ref_k, 'sr')
+plt.plot(ref_lam * 1000, ref_n, 'ob', markersize=2)
+plt.plot(ref_lam * 1000, -ref_k, 'sr', markersize=2)
 plt.title("Gold Spheres 200nm diameter")
-plt.xticks([])
 plt.text(700, 1, "real refractive index", color='blue')
 plt.text(1100, -6, "imaginary refractive index", color='red')
 
 plt.subplot(312)
-plt.plot(ref_lam * 1000, 1000 / mu_a, 'ob')
+plt.plot(ref_lam * 1000, 1000 / mu_a, 'ob', markersize=2)
 plt.ylabel("Absorption Depth [nm]")
 
 plt.subplot(313)
-plt.plot(ref_lam * 1000, abs_cross_section, 'ob')
-plt.plot(ref_lam * 1000, sca_cross_section, 'sr')
+plt.plot(ref_lam * 1000, abs_cross_section, 'ob', markersize=2)
+plt.plot(ref_lam * 1000, sca_cross_section, 'sr', markersize=2)
 plt.xlabel("Wavelength (nm)")
-plt.ylabel("Cross Section (µm²)")
+plt.ylabel("Cross Section [µm²]")
 plt.text(700, 0.01, "absorption", color='blue')
 plt.text(750, 0.1, "scattering", color='red')
-plt.savefig("04_plot.png")
-# plt.show()
+# plt.savefig("../../docs/04_plot.svg")
+plt.show()
