@@ -1,9 +1,3 @@
-# pylint: disable=invalid-name
-# pylint: disable=unused-argument
-# pylint: disable=too-many-locals
-# pylint: disable=no-member
-# pylint: disable=bare-except
-
 """
 Mie scattering calculations for perfect spheres JITTED!.
 
@@ -43,14 +37,15 @@ from numba import njit, int32, float64, complex128
 
 __all__ = ('ez_mie',
            'ez_intensities',
-           'generate_mie_costheta',
            'i_par',
            'i_per',
            'i_unpolarized',
            'mie',
            'mie_S1_S2',
+           'mie_phase_matrix',
            'mie_cdf',
            'mie_mu_with_uniform_cdf',
+           'generate_mie_costheta',
            )
 
 
@@ -145,8 +140,8 @@ def _D_calc(m, x, N):
     return D
 
 
-@njit((complex128, float64, complex128[:], complex128[:]), cache=True)
-def _mie_An_Bn(m, x, a, b):
+@njit((complex128, float64), cache=True)
+def _mie_An_Bn(m, x):
     """
     Compute arrays of Mie coefficients A and B for a sphere.
 
@@ -159,14 +154,17 @@ def _mie_An_Bn(m, x, a, b):
         x: the size parameter of the sphere
 
     Returns:
-        An, Bn: arrays of Mie coefficents
+        a, b: arrays of Mie coefficents An and Bn
     """
+    nstop = int(x + 4.05 * x**0.33333 + 2.0) + 1
+    a = np.zeros(nstop - 1, dtype=np.complex128)
+    b = np.zeros(nstop - 1, dtype=np.complex128)
+
     psi_nm1 = np.sin(x)                   # nm1 = n-1 = 0
     psi_n = psi_nm1 / x - np.cos(x)       # n = 1
     xi_nm1 = complex(psi_nm1, np.cos(x))
     xi_n = complex(psi_n, np.cos(x) / x + np.sin(x))
 
-    nstop = len(a)
     if m.real > 0.0:
         D = _D_calc(m, x, nstop + 1)
 
@@ -191,9 +189,11 @@ def _mie_An_Bn(m, x, a, b):
             psi_nm1 = psi_n
             psi_n = xi_n.real
 
+    return a, b
+
 
 @njit((complex128, float64), cache=True)
-def _small_conducting_mie(m, x):
+def _small_conducting_mie(_m, x):
     """
     Calculate the efficiencies for a small conducting spheres.
 
@@ -201,7 +201,7 @@ def _small_conducting_mie(m, x):
     m.real == 0
 
     Args:
-        m: the complex index of refraction of the sphere
+        _m: the complex index of refraction of the sphere (unused)
         x: the size parameter of the sphere
 
     Returns:
@@ -303,10 +303,7 @@ def _mie_scalar(m, x):
         qext, qsca, qback, g = _small_mie(m, x)
 
     else:
-        nstop = int(x + 4.05 * x**0.33333 + 2.0) + 1
-        a = np.zeros(nstop - 1, dtype=np.complex128)
-        b = np.zeros(nstop - 1, dtype=np.complex128)
-        _mie_An_Bn(m, x, a, b)
+        a, b = _mie_An_Bn(m, x)
 
         nmax = len(a)
         n = np.arange(1, nmax + 1)
@@ -383,96 +380,105 @@ def mie(m, x):
     return qext, qsca, qback, g
 
 
-@njit((complex128, float64, float64[:]), cache=True)
-def _small_mie_conducting_S1_S2(m, x, mu):
+@njit((complex128, float64, int32), cache=True)
+def normalization_factor(m, x, norm_int):
     """
-    Calculate the scattering amplitudes for small conducting spheres.
-
-    The spheres are small perfectly conducting (reflecting) spheres (x<0.1).
-    The amplitude functions have been normalized so that when integrated
-    over all 4𝜋 solid angles, the integral will be qext(𝜋x²).
-
-    The units are weird, sr**(-0.5)
+    Figure out scattering function normalization.
 
     Args:
         m: the complex index of refraction of the sphere
-        x: the size parameter of the sphere
-        mu: the angles, cos(theta), to calculate scattering amplitudes
+        x: dimensionless sphere size
+        norm_int: integer that specifies normalization
 
     Returns:
-        S1, S2: the scattering amplitudes at each angle mu [sr**(-0.5)]
+        scaling factor needed for scattering function
     """
-    ahat1 = 2j / 3 * (1 - 0.2 * x**2) / (1 - 0.5 * x**2 + 2j / 3 * x**3)
-    bhat1 = 1j / 3 * (0.1 * x**2 - 1) / (1 + 0.5 * x**2 - 1j / 3 * x**3)
-    ahat2 = 1j / 30 * x**2
-    bhat2 = -1j * x**2 / 45
+    norm = None
 
-    S1 = 1.5 * x**3 * (ahat1 + bhat1 * mu
-                       + 5 / 3 * ahat2 * mu
-                       + 5 / 3 * bhat2 * (2 * mu**2 - 1))
+    # Qsca normalization
+    if norm_int == 3:
+        norm = x * np.sqrt(np.pi)
 
-    S2 = 1.5 * x**3 * (bhat1 + ahat1 * mu
-                       + 5 / 3 * bhat2 * mu
-                       + 5 / 3 * ahat2 * (2 * mu**2 - 1))
+    # Bohren Normalization
+    elif norm_int == 5:
+        norm = 0.5
 
-    qext = x**4 * (6 * np.abs(ahat1)**2
-                   + 6 * np.abs(bhat1)**2
-                   + 10 * np.abs(ahat2)**2
-                   + 10 * np.abs(bhat2)**2)
+    # Wiscombe Normalization
+    elif norm_int == 6:
+        norm = 1
 
-    norm = np.sqrt(qext * np.pi * x**2)
-    S1 /= norm
-    S2 /= norm
+    else:
+        # calculate qsca and qext
+        qext, qsca, _, _ = _mie_scalar(m, x)
 
-    return [S1, S2]
+        # albedo Normalization
+        if norm_int == 0:
+            norm = x * np.sqrt(np.pi * qext)
+
+        # Unity normalization
+        elif norm_int == 1:
+            norm = x * np.sqrt(qsca * np.pi)
+
+        # 4pi Normalization
+        elif norm_int == 2:
+            norm = x * np.sqrt(qsca / 4)
+
+        # Qext Normalization
+        elif norm_int == 4:  # 4pi
+            norm = x * np.sqrt(qsca * np.pi / qext)
+
+    if norm is None:
+        raise ValueError("norm-int must be in the range 0..6")
+
+    return norm
 
 
-@njit((complex128, float64, float64[:]), cache=True)
-def _small_mie_S1_S2(m, x, mu):
+def norm_string_to_integer(s):
     """
-    Calculate the scattering amplitude functions for small spheres (x<0.1).
+    Encode normalization choice as an integer.
 
-    The amplitude functions have been normalized so that when integrated
-    over all 4*pi solid angles, the integral will be qext*pi*x**2.
-
-    The units are weird, sr**(-0.5)
+    This is needed because these string operations cannot be
+    done in a jitted function under numba.  We cannot use enums for
+    the same reason!
 
     Args:
-        m: the complex index of refraction of the sphere
-        x: the size parameter of the sphere
-        mu: the angles, cos(theta), to calculate scattering amplitudes
+        s: string describing normalization desired.
 
     Returns:
-        S1, S2: the scattering amplitudes at each angle mu [sr**(-0.5)]
+        integer used in _mie_S1_S2() determine normalization
     """
-    m2 = m * m
-    m4 = m2 * m2
-    x2 = x * x
-    x3 = x2 * x
-    x4 = x2 * x2
+    ii = None
+    norm = s.lower()
 
-    D = m2 + 2 + (1 - 0.7 * m2) * x2
-    D -= (8 * m4 - 385 * m2 + 350) * x4 / 1400.0
-    D += 2j * (m2 - 1) * x3 * (1 - 0.1 * x2) / 3
-    ahat1 = 2j * (m2 - 1) / 3 * (1 - 0.1 * x2 + (4 * m2 + 5) * x4 / 1400) / D
-    bhat1 = 1j * x2 * (m2 - 1) / 45 * (1 + (2 * m2 - 5) / 70 * x2)
-    bhat1 /= 1 - (2 * m2 - 5) / 30 * x2
-    ahat2 = 1j * x2 * (m2 - 1) / 15 * (1 - x2 / 14)
-    ahat2 /= 2 * m2 + 3 - (2 * m2 - 7) / 14 * x2
+    if norm in ['a', 'albedo']:
+        ii = 0
 
-    S1 = 1.5 * x3 * (ahat1 + bhat1 * mu + 5 / 3 * ahat2 * mu)
-    S2 = 1.5 * x3 * (bhat1 + ahat1 * mu + 5 / 3 * ahat2 * (2 * mu**2 - 1))
+    elif norm in ['1', 'one', 'unity']:
+        ii = 1
 
-    # norm = sqrt(qext*pi*x**2)
-    norm = np.sqrt(np.pi * 6 * x**3 * (ahat1 + bhat1 + 5 * ahat2 / 3).real)
-    S1 /= norm
-    S2 /= norm
+    elif norm in ['four_pi', '4pi']:
+        ii = 2
 
-    return [S1, S2]
+    elif norm in ['qsca', 'scattering_efficiency']:
+        ii = 3
+
+    elif norm in ['qext', 'extinction_efficiency']:
+        ii = 4
+
+    elif norm in ['bohren']:
+        ii = 5
+
+    elif norm in ['wiscombe']:
+        ii = 6
+
+    if ii is None:
+        raise ValueError("normalization must be one of 'albedo' (default), 'one'"
+                         "'4pi', 'qext', 'qsca', 'bohren', or 'wiscombe'")
+    return ii
 
 
-@njit((complex128, float64, float64[:]), cache=True)
-def _mie_S1_S2(m, x, mu):
+@njit((complex128, float64, float64[:], int32), cache=True)
+def _mie_S1_S2(m, x, mu, norm_int):
     """
     Calculate the scattering amplitude functions for spheres.
 
@@ -485,14 +491,12 @@ def _mie_S1_S2(m, x, mu):
         m: the complex index of refraction of the sphere
         x: the size parameter of the sphere
         mu: array of angles, cos(theta), to calculate scattering amplitudes
+        norm_int: integer describing type of normalization
 
     Returns:
         S1, S2: the scattering amplitudes at each angle mu [sr**(-0.5)]
     """
-    nstop = int(x + 4.05 * x**0.33333 + 2.0) + 1
-    a = np.zeros(nstop - 1, dtype=np.complex128)
-    b = np.zeros(nstop - 1, dtype=np.complex128)
-    _mie_An_Bn(m, x, a, b)
+    a, b = _mie_An_Bn(m, x)
 
     nangles = len(mu)
     S1 = np.zeros(nangles, dtype=np.complex128)
@@ -504,33 +508,28 @@ def _mie_S1_S2(m, x, mu):
         pi_nm1 = 1
         for n in range(1, nstop):
             tau_nm1 = n * mu[k] * pi_nm1 - (n + 1) * pi_nm2
-
-            S1[k] += (2 * n + 1) * (pi_nm1 * a[n - 1]
-                                    + tau_nm1 * b[n - 1]) / (n + 1) / n
-
-            S2[k] += (2 * n + 1) * (tau_nm1 * a[n - 1]
-                                    + pi_nm1 * b[n - 1]) / (n + 1) / n
-
+            S1[k] += (2 * n + 1) * (pi_nm1 * a[n - 1] + tau_nm1 * b[n - 1]) / (n + 1) / n
+            S2[k] += (2 * n + 1) * (tau_nm1 * a[n - 1] + pi_nm1 * b[n - 1]) / (n + 1) / n
             temp = pi_nm1
             pi_nm1 = ((2 * n + 1) * mu[k] * pi_nm1 - (n + 1) * pi_nm2) / n
             pi_nm2 = temp
 
-    # calculate norm = sqrt(pi * Qext * x**2)
-    n = np.arange(1, nstop + 1)
-    norm = np.sqrt(2 * np.pi * np.sum((2 * n + 1) * (a.real + b.real)))
+    normalization = normalization_factor(m, x, norm_int)
 
-    S1 /= norm
-    S2 /= norm
+    S1 /= normalization
+    S2 /= normalization
 
     return [S1, S2]
 
 
-def mie_S1_S2(m, x, mu):
+def mie_S1_S2(m, x, mu, norm='albedo'):
     """
     Calculate the scattering amplitude functions for spheres.
 
-    The amplitude functions have been normalized so that when integrated
-    over all 4*pi solid angles, the integral will be qext*pi*x**2.
+    The normalization is controlled by `norm` and should be one of
+    ['albedo', 'one', '4pi', 'qext', 'qsca', 'bohren', or 'wiscombe']
+    The normalization describes the integral of the scattering phase
+    function over all 4𝜋 steradians.
 
     The units are weird, sr**(-0.5)
 
@@ -538,16 +537,70 @@ def mie_S1_S2(m, x, mu):
         m: the complex index of refraction of the sphere
         x: the size parameter of the sphere
         mu: cos(theta) or array of angles [cos(theta_i)]
+        norm: (optional) string describing scattering function normalization
 
     Returns:
         S1, S2: the scattering amplitudes at each angle mu [sr**(-0.5)]
     """
+    norm_int = norm_string_to_integer(norm)
     if np.isscalar(mu):
         mu_array = np.array([mu], dtype=float)
-        s1, s2 = _mie_S1_S2(m, x, mu_array)
+        s1, s2 = _mie_S1_S2(m, x, mu_array, norm_int)
         return s1[0], s2[0]
 
-    return _mie_S1_S2(m, x, mu)
+    return _mie_S1_S2(m, x, mu, norm_int)
+
+
+def mie_phase_matrix(m, x, mu, norm='albedo'):
+    """
+    Calculate the scattering (Mueller) matrix.
+
+    If mu has length N, then the returned matrix is 4x4xN.  If mu is a scalar
+    then the matrix is 4x4
+
+    The phase scattering matrix is computed from the scattering amplitude
+    functions, according to equations 5.2.105-6 in K. N. Liou (**2002**) -
+    *An Introduction to Atmospheric Radiation*, Second Edition.
+
+    or
+
+    Bohren and Huffman, *Absorption and Scattering of Light by Small Particles*,
+    JOHN WILEY & SONS, page 112, (1983).
+
+    The normalization is controlled by `norm` and should be one of
+    ['albedo', 'one', '4pi', 'qext', 'qsca', 'bohren', or 'wiscombe']
+    The normalization describes the integral of the scattering phase
+    function over all 4𝜋 steradians.
+
+    Args:
+        m: the complex index of refraction of the sphere
+        x: the size parameter of the sphere
+        mu: the angles, cos(theta), for the phase scattering matrix
+        norm: (optional) string describing scattering function normalization
+
+    Returns:
+        p: the phase scattering matrix [sr**(-1.0)]
+    """
+    s1, s2 = mie_S1_S2(m=m, x=x, mu=mu, norm=norm)
+
+    mu = np.atleast_1d(mu)
+    s1_star = np.conjugate(s1)
+    s2_star = np.conjugate(s2)
+    m1 = (s1 * s1_star).real
+    m2 = (s2 * s2_star).real
+    s21 = (0.5 * (s1 * s2_star + s2 * s1_star)).real
+    d21 = (-0.5j * (s1 * s2_star - s2 * s1_star)).real
+    phase = np.zeros(shape=(4, 4, mu.size))
+    phase[0, 0] = 0.5 * (m2 + m1)
+    phase[0, 1] = 0.5 * (m2 - m1)
+    phase[1, 0] = phase[0, 1]
+    phase[1, 1] = phase[0, 0]
+    phase[2, 2] = s21
+    phase[2, 3] = -d21
+    phase[3, 2] = d21
+    phase[3, 3] = s21
+
+    return phase.squeeze()
 
 
 def mie_cdf(m, x, num):
@@ -573,17 +626,8 @@ def mie_cdf(m, x, num):
         cdf: array of cumulative distribution function values
     """
     mu = np.linspace(-1, 1, num)
-    s1, s2 = mie_S1_S2(m, x, mu)
-
-    s = (np.abs(s1)**2 + np.abs(s2)**2) / 2
-
-    cdf = np.zeros(num)
-    total = 0
-    for i in range(num):
-        # need the extra 2pi because scattering normalized over 4π steradians
-        total += s[i] * 2 * np.pi * (2 / num)
-        cdf[i] = total
-
+    intensity_per_mu = i_unpolarized(m, x, mu, norm='4pi') / num
+    cdf = np.cumsum(intensity_per_mu)
     return mu, cdf
 
 
@@ -655,7 +699,7 @@ def generate_mie_costheta(mu_cdf):
     Args:
        mu_cdf: a cumulative distribution function
 
-    Returns
+    Returns:
        The cosine of the scattering angle
     """
     # the following should be equivalent to these four lines
@@ -671,7 +715,7 @@ def generate_mie_costheta(mu_cdf):
     return x
 
 
-def i_per(m, x, mu):
+def i_per(m, x, mu, norm='albedo'):
     """
     Return the scattered intensity in a plane normal to the incident light.
 
@@ -680,20 +724,26 @@ def i_per(m, x, mu):
     that the integral of the unpolarized intensity over 4π steradians
     is equal to the single scattering albedo.
 
-    Args:
-       m: the complex index of refraction of the sphere
-       x: the size parameter of the sphere
-       mu: the angles, cos(theta), to calculate intensities
+    The normalization is controlled by `norm` and should be one of
+    ['albedo', 'one', '4pi', 'qext', 'qsca', 'bohren', or 'wiscombe']
+    The normalization describes the integral of the scattering phase
+    function over all 4𝜋 steradians.
 
-    Returns
-       The intensity at each angle in the array mu.  Units [1/sr]
+    Args:
+        m: the complex index of refraction of the sphere
+        x: the size parameter of the sphere
+        mu: the angles, cos(theta), to calculate intensities
+        norm: (optional) string describing scattering function normalization
+
+    Returns:
+        The intensity at each angle in the array mu.  Units [1/sr]
     """
-    s1, _ = mie_S1_S2(m, x, mu)
+    s1, _ = mie_S1_S2(m, x, mu, norm)
     intensity = np.abs(s1)**2
     return intensity.astype('float')
 
 
-def i_par(m, x, mu):
+def i_par(m, x, mu, norm='albedo'):
     """
     Return the scattered intensity in a plane parallel to the incident light.
 
@@ -702,37 +752,48 @@ def i_par(m, x, mu):
     that the integral of the unpolarized intensity over 4π steradians
     is equal to the single scattering albedo.
 
-    Args:
-       m: the complex index of refraction of the sphere
-       x: the size parameter
-       mu: the cos(theta) of each direction desired
+    The normalization is controlled by `norm` and should be one of
+    ['albedo', 'one', '4pi', 'qext', 'qsca', 'bohren', or 'wiscombe']
+    The normalization describes the integral of the scattering phase
+    function over all 4𝜋 steradians.
 
-    Returns
-       The intensity at each angle in the array mu.  Units [1/sr]
+    Args:
+        m: the complex index of refraction of the sphere
+        x: the size parameter of the sphere
+        mu: the angles, cos(theta), to calculate intensities
+        norm: (optional) string describing scattering function normalization
+
+    Returns:
+        The intensity at each angle in the array mu.  Units [1/sr]
     """
-    _, s2 = mie_S1_S2(m, x, mu)
+    _, s2 = mie_S1_S2(m, x, mu, norm)
     intensity = np.abs(s2)**2
     return intensity.astype('float')
 
 
-def i_unpolarized(m, x, mu):
+def i_unpolarized(m, x, mu, norm='albedo'):
     """
     Return the unpolarized scattered intensity at specified angles.
 
     This is the average value for randomly polarized incident light.
-    The intensity is normalized such
-    that the integral of the unpolarized intensity over 4π steradians
-    is equal to the single scattering albedo.
+    The intensity is normalized such that the integral of the unpolarized
+    intensity over 4π steradians is equal to the single scattering albedo.
+
+    The normalization is controlled by `norm` and should be one of
+    ['albedo', 'one', '4pi', 'qext', 'qsca', 'bohren', or 'wiscombe']
+    The normalization describes the integral of the scattering phase
+    function over all 4𝜋 steradians.
 
     Args:
-       m: the complex index of refraction of the sphere
-       x: the size parameter
-       mu: the cos(theta) of each direction desired
+        m: the complex index of refraction of the sphere
+        x: the size parameter of the sphere
+        mu: the angles, cos(theta), to calculate intensities
+        norm: (optional) string describing scattering function normalization
 
-    Returns
-       The intensity at each angle in the array mu.  Units [1/sr]
+    Returns:
+        The intensity at each angle in the array mu.  Units [1/sr]
     """
-    s1, s2 = mie_S1_S2(m, x, mu)
+    s1, s2 = mie_S1_S2(m, x, mu, norm)
     intensity = (np.abs(s1)**2 + np.abs(s2)**2) / 2
     return intensity.astype('float')
 
@@ -745,20 +806,20 @@ def ez_mie(m, d, lambda0, n_env=1.0):
         m: the complex index of refraction of the sphere [-]
         d: the diameter of the sphere                    [same units as lambda0]
         lambda0: wavelength in a vacuum                  [same units as d]
-        n_env: real index of medium around sphere        [-]
+        n_env: real index of medium around sphere, optional.
 
     Returns:
-        qext: the total extinction efficiency                  [-]
-        qsca: the scattering efficiency                        [-]
-        qback: the backscatter efficiency                      [-]
-        g: the average cosine of the scattering phase function [-]
+        qext: the total extinction efficiency                  [-].
+        qsca: the scattering efficiency                        [-].
+        qback: the backscatter efficiency                      [-].
+        g: the average cosine of the scattering phase function [-].
     """
     m_env = m / n_env
     x_env = np.pi * d / (lambda0 / n_env)
     return mie(m_env, x_env)
 
 
-def ez_intensities(m, d, lambda0, mu, n_env=1.0):
+def ez_intensities(m, d, lambda0, mu, n_env=1.0, norm='albedo'):
     """
     Return the scattered intensities from a sphere.
 
@@ -771,12 +832,18 @@ def ez_intensities(m, d, lambda0, mu, n_env=1.0):
 
     The unpolarized scattering is the average of the two scattered intensities.
 
+    The normalization is controlled by `norm` and should be one of
+    ['albedo', 'one', '4pi', 'qext', 'qsca', 'bohren', or 'wiscombe']
+    The normalization describes the integral of the scattering phase
+    function over all 4𝜋 steradians.
+
     Args:
         m: the complex index of refraction of the sphere [-]
         d: the diameter of the sphere                    [same units as lambda0]
         lambda0: wavelength in a vacuum                  [same units as d]
         mu: the cos(theta) of each direction desired     [-]
-        n_env: real index of medium around sphere        [-]
+        n_env: real index of medium around sphere, optional
+        norm: (optional) string describing scattering function normalization
 
     Returns:
         ipar, iper: scattered intensity in parallel and perpendicular planes [1/sr]
@@ -784,7 +851,7 @@ def ez_intensities(m, d, lambda0, mu, n_env=1.0):
     m_env = m / n_env
     lambda_env = lambda0 / n_env
     x_env = np.pi * d / lambda_env
-    s1, s2 = mie_S1_S2(m_env, x_env, mu)
+    s1, s2 = mie_S1_S2(m_env, x_env, mu, norm)
     ipar = np.abs(s2)**2
     iper = np.abs(s1)**2
     Ipar = ipar.astype('float')
